@@ -2,8 +2,8 @@
 ;;
 ;; Author: Lars Schmidt-Thieme <schmidt-thieme@ismll.de>
 ;; Maintainer: Lars Schmidt-Thieme <schmidt-thieme@ismll.de>
-;; Package-Version: 20251126.2007
-;; Package-Revision: aed6438f2278
+;; Package-Version: 20251208.2300
+;; Package-Revision: cbef68f873f0
 ;; Package-Requires: ((emacs "28.1") (compat "30.1.0.1"))
 ;; Keywords: frames, processes, convenience
 ;; URL: https://github.com/lsth/dwin
@@ -11,486 +11,121 @@
 ;;
 ;;; Commentary:
 ;;
-;; Provides two types of navigation:
-;; 1. 🏷️ navigation by name,
+;; dwin enables interaction with desktop windows, i.e., windows of
+;; external applications such as Firefox or okular, managed
+;; by an external X11 window manager or Wayland compositor
+;; (the latter currently only for KDE KWin).
+;;
+;; Provides three types of desktop window interactions:
+;; 1. ⧉ arranging desktop windows from within Emacs with keys,
+;;    e.g., move windows around, resize them etc.,
+;; 2. 🏷️ navigation by name,
 ;;    e.g., switching to firefox, zotero, back to Emacs, and
-;; 2. 🔀 directional navigation,
-;;    e.g., moving to the window to the right or below, as well as
-;; 3. ⧉ arranging desktop windows from within Emacs with keys.
+;; 3. 🔀 directional navigation,
+;;    e.g., moving to the window to the right or below.
 ;;
-;; That navigation works globally, also outside Emacs, requires that
-;; the window manager forwards some keys globally to Emacs;
-;; for KDE one can bind a one-line script that uses Emacsclient
-;; to forward the key to Emacs, using `dwin-input-key' defined in
-;; sect.  3. See etc/bin/dwin-firefox for an example.  See
-;; Further Details / 1 for why we do not use tools like qdotool for
-;; sending keys.
+;; Install from MELPA with
 ;;
-;; Emacs then can use functions provided by the window manager
-;; to implement window navigation globally.  We captured all required
+;;    (use-package dwin
+;;      :config
+;;      (dwin-setup))
+;;
+;; ⧉ arrange desktop windows with `dwin-grab', i.e., move them
+;; around, resize them, etc.
+;;
+;; 🏷️ Navigation by name is done with `dwin-switch-to-app' that will
+;; i) start an app, if it has no window yet,
+;; ii) switch to its window, if it is not active, or
+;; iii) switch back to Emacs, if it is already active (toggle).
+;; For the latter, switching back to Emacs from outside Emacs,
+;; a shortcut command for toggling the application has to be
+;; bound desktop global by the window manager / compositor,
+;; e.g., via
+;;
+;;   (defun my/firefox (&optional prefix)
+;;     (interactive (list current-prefix-arg))
+;;     (dwin-switch-to-app "firefox" prefix))
+;;   (dwin-keymap-desktopglobal-set "<f11>" #'my/firefox)
+;;
+;; By default, navigation by name will switch to the first window of
+;; an application, if it has several.  You can use a prefix arg to
+;; switch to a specific one, e.g., C-2 M-x my/firefox or C-2 <f11> to
+;; switch to the second one.
+;;
+;; To switch back from outside Emacs to Emacs from any other window,
+;; bind `dwin-switch-to-emacs-or' to a desktop global key:
+;;
+;;   (dwin-keymap-desktopglobal-set "C-<f11>" #'dwin-switch-to-emacs-or)
+;;
+;;
+;; 🔀 Directional navigation between desktop windows and Emacs
+;; windows is accomplished by `dwin-windmove-left' that has to be
+;; bound to desktop global keys the same way as above:
+;;
+;;  (dwin-keymap-desktopglobal-set "M-<left>"  #'dwin-windmove-left)
+;;  (dwin-keymap-desktopglobal-set "M-<right>" #'dwin-windmove-right)
+;;  (dwin-keymap-desktopglobal-set "M-<up>"    #'dwin-windmove-up)
+;;  (dwin-keymap-desktopglobal-set "M-<down>"  #'dwin-windmove-down)
+;;
+;; These functions try to move inside Emacs via windmove, and if
+;; this fails, use the window manager to move out of Emacs.
+;; The same method also uses the window manager to move
+;; directional from desktop windows to other desktop windows
+;; or back to Emacs.
+;;
+;; How it is implemented
+;;
+;; dwin enables Emacs to use functions provided by the window manager
+;; to implement window interactions.  We captured all required
 ;; methods in a window manager proxy object `dwin-proxy' whose
 ;; methods can be called via `dwin-call'.  The proxy has to be created
 ;; once before use, e.g., during Emacs initialization, using
-;; `dwin-setup'.  Currently only a proxy for KDE is implemented.
-;; But it should be possible to implement further ones.
+;; `dwin-setup'.  Currently two proxies are implemented:
+;; - one for generic X11 window managers and
+;; - one for KDE/KWin on Wayland or X11.
+;; It should be possible to implement further ones.
+;; The X11 proxy uses xdotool.
 ;; The KDE proxy uses
 ;; i) dbus calls to org.kde.kglobalaccel (KDE's shortcuts
 ;;    application), esp.  for directional navigation, and
 ;; ii) kdotool for navigation by name.
-;; See sect. 4 below.
+;; See file dwin-kwin.el.
 ;;
-;; 🏷️ Navigation by name is provided by `dwin-switch-to-app' that will
-;; i) start an app, if it has no window yet,
-;; ii) switch to its window, if it is not active, or
-;; iii) switch back to Emacs, if it is already active (toggle).
-;; For each app (firefox, zotero etc.) one needs to
-;; i) define one Emacs command (e.g., `my/firefox'),
-;; ii) bind it to a key in Emacs and
-;; iii) make sure this key is forwarded globally to Emacs,
-;;   i.e.,
-;;   a) create a small script (e.g., etc/bin/dwin-firefox) and
-;;   b) bind it globally to this key.
-;; Switching back to Emacs is handled by `dwin-switch-to-emacs-or'.
-;; It needs the same handling as the other apps above.
-;; By default, navigation by name will switch to the first window of
-;; an application, if it has several.  You can use a prefix arg to
-;; switch to a specific one, e.g., C-2 M-x my/firefox or C-2 <f11> to
-;; switch to the second one.  See sect. 5 below.
+;; That navigation works globally, also outside Emacs, requires that
+;; the window manager forwards some keys globally to Emacs;
+;; for KDE one can bind a one-line script _emacs-key that uses
+;; Emacsclient to forward the key to Emacs, using `dwin-input-key'.
+;; The KWin keybinding is accomplished by
+;; `dwin-sync-desktopglobal-keys' (which is triggered automatically
+;; when you set desktop global keys with `dwin-keymap-desktopglobal-set').
+;; It assembles a brief .desktop application file and puts it into
+;; KWin's application directory ~/.local/share/applications, where
+;; KWin will take it up automatically.
 ;;
-;; For 🔀 directional navigation, we defined a short function
-;; `dwin-windmove-left' for each direction.  The function tries to
-;; move inside Emacs via windmove, and if this fails, uses the
-;; window manager to move out of Emacs.
-;; The same method also uses the window manager to move
-;; directional from desktop windows.  See sect.  6.
+;; See etc/example-emacs-inits/01-dwin-via-package/init.el for an
+;; example configuration.
 ;;
-;; Sect.  7 contains function `dwin-grab' to ⧉ arrange desktop
-;; windows, i.e., to resize them, reposition them etc.
-;;
-;; See etc/example-emacs-init/init.el for an example configuration.
-;;
-;; Known Issues and Limitations:
-;; 1. Requesting help for a **global** key binding with `describe key'
-;;    will not work.  Emacs will get the key forwarded, but not as
-;;    input for the already running describe-key function.  In effect,
-;;    if you say M-x describe-key and then type a global key like
-;;    M-left, its action is executed and help for the **next** key the
-;;    user types is shown.  To view the help, you have to know how the
-;;    key is called in Emacs and say M-: (describe-key (kbd
-;;    "M-<left>")) instead.  One could instrument `dwin-input-key' to
-;;    detect if describe-key is running by checking
-;;    (describe-key (kbd "M-<left>")) and then run describe-key with
-;;    the key instead of the command the key is bound to.  However, I
-;;    found no way to cancel the already running describe-key
-;;    command.  So the second issue will persist.
-;;
-;; Further details:
-;; 1. Why cannot we just use ydotool to send keys to Emacs?
-;;    Tools like ydotool seem to be able to send key events only to
-;;    the active/focused window and would be able only to implement
-;;    the cases where one wants to move from within Emacs, not the
-;;    cases where one wants to move back to Emacs from other
-;;    applications (or between other applications).  One could
-;;    reimplement dwin in bash, then there is no need to input keys
-;;    into Emacs anymore.
+;; Custom variables are in dwin-core.el.
 ;; 
-;;    Unfortunately sending dbus events to kwin from bash was unstable
-;;    for me: running
-;;      qdbus6 org.kde.kglobalaccel /component/kwin \
-;;        org.kde.kglobalaccel.Component invokeShortcut
-;;        "Switch Window Left"
-;;    often (not always) yielded
-;;      Cannot find 'org.kde.kglobalaccel.Component' in object
-;;        /component/kwin at org.kde.kglobalaccel
-;;    This never happened when sending the same events from Emacs.
+;; More information on the dwin webpage: https://github.com/lsth/dwin
 
 ;;; Code:
 ;;_ 0. required packages
-(require 'dbus)
-(require 'cl-lib)
-(require 'server)
+(require 'dwin-core)  ;; the abstract base class of a wm proxy
+(require 'dwin-kwin)  ;; an implementation for KDE KWin
 
-(declare-function keymap-set nil)
-(when (version< emacs-version "29.1")  ; 28.2: needs it; 29.1: does not need it
-  (require 'dwin-compat))
+;;_ 1. see dwin-core.el
+;;_ 2. see dwin-core.el
+;;_ 3. see dwin-core.el
 
-;;_ 1. some customization
-(defgroup dwin nil
-  "Customization options for the desktop window manager (dwin) package."
-  :group 'applications  ;; optional: parent group
-  :prefix "dwin-")      ;; optional: used for variables/functions
-
-(defcustom dwin-switch-to-emacs-function 'switch-to-buffer
-  "Function `dwin-switch-to-emacs-or' will call when already in Emacs."
-  :type 'function
-  :group 'dwin)
-
-(defcustom dwin-move-x 100
-  "Default positions to move a window horizonally."
-  :type 'integer
-  :group 'dwin)
-
-(defcustom dwin-move-y 100
-  "Default positions to move a window vertically."
-  :type 'integer
-  :group 'dwin)
-
-(defcustom dwin-resize-x 100
-  "Default positions to resize a window horizontally."
-  :type 'integer
-  :group 'dwin)
-
-(defcustom dwin-resize-y 100
-  "Default positions to resize a window vertically."
-  :type 'integer
-  :group 'dwin)
-
-(defcustom dwin-kwin-use-shortcuts t
-  "If the KDE/KWin proxy should use KWin shortcuts.
-If nil, implements the behavior in elisp.
-To take effect, the proxy has to be reset via `dwin-setup'."
-  :type 'boolean
-  :group 'dwin)
-
-(defcustom dwin-app-startup-grace-period 0.3
-  "Seconds to wait for an app to start to capture new windows."
-  :type 'float
-  :group 'dwin)
-
-(defcustom dwin-log-level 1
-  "How verbose dwin should send messages to the user.
-0 = none, 1 = info, 2 = debug."
-  :type '(integer :tag "Log level")
-  :group 'dwin)
-
-;;_ 2. some prerequisites / utilities
-;;_ 2.1 simple alist objects
-(defun dwin-get (obj attr)
-  "Get attribute ATTR (a symbol) of OBJ (an alist object)."
-  (let ((ref-cons (assoc attr obj)))
-    (if ref-cons
-        (cdr ref-cons)
-      (error "Attribute/method '%s' not found in obj '%s'" attr obj))))
-
-(defun dwin-set (obj attr val)
-  "Set attribute ATTR (a symbol) of OBJ (an alist object) to value VAL."
-  (let ((ref-cons (assoc attr obj)))
-    (if ref-cons
-	(setcdr ref-cons val)
-      (error "Attribute/method '%s' not found in obj '%s'" attr obj))))
-
-(defun dwin-call (obj method &rest args)
-  "Call METHOD (a symbol) on OBJ (an alist object).
-ARGS are passed as arguments to the method function."
-  (let ((fn (dwin-get obj method)))
-    (apply fn args)))
-
-(defun dwin-call-apply (obj method args)
-  "Call METHOD (a symbol) on OBJ (an alist object).
-ARGS is a list of arguments provided to the function.
-This is different from `dwin-call' where args can be
-several arguments that are then captured by a list.
-See the following example that one cannot implement
-with `dwin-call' (without changing lines 1+2), but
-with `dwin-call-apply'.
-Example:
-    (let* ((x (list (cons \='fun (lambda(&rest args) (apply \='+ args)))))
-           (values1 (list 1 2 3)))
-      (dwin-call-apply x \='fun (append values1 (list 4 5)))) ;; 15"
-  (let ((fn (dwin-get obj method)))
-    (apply fn args)))
-
-(defun dwin-extend (self extensions)
-  "Update object SELF with new attributes and functions EXTENSIONS (an alist).
-If an attribute or a method exists, overwrite it (using `dwin-set').
-If an attribute or a method does not exist yet, define it (appending to SELF).
-This function operates in place and will mutate SELF.
-Better than using `dwin-set' directly, because it also will work for new
-attributes and functions."
-  (dolist (pair extensions)
-    (let ((key (car pair))
-          (val (cdr pair)))
-      (let ((cell (assoc key self)))
-        (if cell
-	    ;; a. attribute/method exists: update
-            (dwin-set self key val)
-          ;; b. new attribute/method: append
-          (nconc self (list pair))))))
-  self)
-
-(defalias 'dwin-method #'dwin-get
- "Alias for `dwin-get'. Obsolete. Use `dwin-get' directly.")
-(make-obsolete 'dwin-method 'dwin-get "0.1.2")
-
-;;_ 2.2 logging
-(defun dwin-message (level format &rest args)
-  "Send a `message' with FORMAT and ARGS.
-Skip, if LEVEL exceeds `dwin-log-level'."
-  (when (<= level dwin-log-level)
-    (apply #'message format args)))
-
-;;_ 2.3 functions on basic data structures
-(defun dwin-collect (lst pred)
-  "Return a sublist of LST of all elements x for which (PRED x) is t."
-  (cl-loop for x in lst
-           for r = (funcall pred x)
-           when r collect x))
-
-(defun dwin-range (a &optional b step)
-  "Return a list of integers in a given range.
-If only A is given, return integers from 0 to A-1.
-If only A and B are given, return integers from A to B-1.
-If A, B, and STEP are given, return integers from A to B-1 in steps of STEP."
-  (let* ((start (if b a 0))
-	 (end (if b b a))
-	 (step (or step 1)))
-    (cl-loop for i from start below end by step
-             collect i)))
-
-(defun dwin-argmin-index (lst pred)
-  "Find the index of the element in LST for which PRED returns the smallest value."
-  (cl-loop with best-idx = 0
-	   with best-val = (funcall pred (nth 0 lst))
-	   for x in (cdr lst)
-	   for i from 1
-           for val = (funcall pred x)
-           when (< val best-val)
-	   do (setq best-idx i
-		    best-val val)
-	   finally return best-idx))
-
-(defun dwin-argmin (lst pred)
-  "Find the element in LST for which PRED returns the smallest value."
-  (cl-loop with best-elem = (nth 0 lst)
-	   with best-val = (funcall pred (nth 0 lst))
-	   for x in (cdr lst)
-           for val = (funcall pred x)
-           when (< val best-val)
-	   do (setq best-elem x
-		    best-val val)
-	   finally return best-elem))
-
-(defun dwin-argmax (lst pred)
-  "Find the index of the element in LST for which PRED returns the largest value."
-  (dwin-argmin lst (lambda (x) (- (funcall pred x)))))
-
-;;_ 3. input keys programatically (e.g., from emacsclient)
-;;;###autoload
-(defun dwin-input-key (key)
-  "Input KEY as if typed by the user.
-
-  Useful to send keys from outside Emacs via emacsclient.
-  KEY is a string that `kbd' can understand.
-  Does only work for KEYs bound to commands, not for prefix keys.
-  Uses and consumes a prefix arg currently typed.
-  When using commands that modify buffers from emacsclient,
-  you need to set a `current-buffer', e.g.,
-    emacsclient -e \\'(with-current-buffer \"*scratch*\"
-      (dwin-input-key \"a\"))\\'
-
-  Example: (dwin-input-key \"C-<f11>\")."
-  (interactive (list
-		(read-from-minibuffer "Key to input: ")))
-  (dwin-message 2 "dwin-input-key: %s" key)
-  (if (eq this-command 'describe-key)
-      ;; 1. the user is trying to get help for the key: show it
-      (progn
-	(describe-key (kbd key))
-	(message "describe-help still active. Say C-g to cancel."))
-    ;; 2. otherwise: execute the command bound to the key
-    (let* ((key-vector (read-kbd-macro key))
-	   (last-command-event (aref key-vector
-				     (- (length key-vector) 1)))
-					; then self-insert-command
-					; will work
-	   (cmd (key-binding (kbd key))))
-      (if cmd
-	  ;; `command-execute' uses prefix arg, `call-interactively'
-	  ;; does not !
-	  (command-execute cmd nil key-vector)
-	(message "no key binding for %s." key) ))))
-
-;;_ 4. window manager proxy, here for X11-generic and KDE/KWin
+;;_ 4. window manager proxy, here x11 generic
 ;; we define three object constructors for window manager proxies:
 ;; 1. proxy-dotool-based: to pool all methods common for xdotool and kdotool.
+;;    - in dwin-core.el
 ;; 2. proxy-x11-generic: to talk to x11 via xdotool, not to any wm specifically.
+;;    - here
 ;; 3. proxy-kwin: to talk to kwin via kdotool and dbus.
-
-;;_ 4.1 common wm proxy methods for dotool based interaction (xdotool, kdotool)
-(defun dwin--next-window-in-direction (proxy direction &optional window)
-  "Get the next window from WINDOW in DIRECTION.
-DIRECTION can be \='left, \='right, \='up or \='down.
-If WINDOW is missing, the active window is taken as starting point.
-PROXY is the window manager proxy.
-TODO: break ties."
-  (dwin-message 2 "next window in direction %s" direction)
-  (let* ((win-start (or window (dwin-call proxy 'getactivewindow)))
-	 (desktop (dwin-call proxy 'get_desktop_for_window win-start))
-	 (wins (remove win-start
-		       (dwin-call proxy 'search "--desktop" (format "%s" desktop) "")))
-	 (geom-start (dwin-call proxy 'getwindowgeometry win-start))
-	 (geoms (mapcar (lambda (win) (dwin-call proxy 'getwindowgeometry win)) wins))
-	 ;;
-	 (pred-filter (pcase direction
-			(`left  (lambda (idx) (< (nth 0 (nth idx geoms)) (nth 0 geom-start))))
-			(`right (lambda (idx) (> (nth 0 (nth idx geoms)) (nth 0 geom-start))))
-			(`up    (lambda (idx) (< (nth 1 (nth idx geoms)) (nth 1 geom-start))))
-			(`down  (lambda (idx) (> (nth 1 (nth idx geoms)) (nth 1 geom-start))))))
-	 (pred-score (pcase direction
-		       (`left  (lambda (idx) (- (nth 0 (nth idx geoms)))))
-		       (`right (lambda (idx) (nth 0 (nth idx geoms))))
-		       (`up    (lambda (idx) (- (nth 1 (nth idx geoms)))))
-		       (`down  (lambda (idx) (nth 1 (nth idx geoms))))))
-	 (indices-qual (dwin-collect (dwin-range (length wins)) pred-filter))
-	 (index-best   (when indices-qual (dwin-argmin indices-qual pred-score))) )
-    (dwin-message 2 "geom-start: %s" geom-start)
-    (dwin-message 2 "geoms: %s" geoms)
-    (dwin-message 2 "indices-qual: %s" indices-qual)
-    (dwin-message 2 "index-best: %s" index-best)
-
-    (when index-best (nth index-best wins))))
-
-(defun dwin--switch-direction (proxy direction)
-  "Switch from active window in DIRECTION using PROXY."
-  (dwin-message 2 "dwin--switch-direction: %s" direction)
-  (let ((win (dwin--next-window-in-direction proxy direction)))
-    (when win
-      (dwin-call proxy 'windowactivate win))))
-
-(defun dwin--new-proxy-dotool-based ()
-  "Create new (abstract) base object for dotool based proxies."
-  (let ((self nil))  ;; forward reference for self
-    (setq self
-           (list
-	    ;; private attributes
-	    (cons '_class "proxy-dotool-based")
-	    (cons 'dotool-name "xdotool")
-	    ;; private methods:
-	    ;; yields a list of lines
-	  ;; xdotool and kdotool >= 0.2.2 signalsexitcode 1 on empty search results. avoid propagating an error.
-	  (cons 'dotool (lambda (&rest args)
-			  (let ((exitcode))
-                            (condition-case err
-				(let*
-				    ((lines (apply #'process-lines-handling-status
-						   (dwin-get self 'dotool-name)
-						   (lambda (status) (setq exitcode status))
-						   args)))
-				  (if (or (zerop exitcode)
-					  (null lines)) ; also OK if no output (empty search results)
-				      lines
-				    (error (message
-					    "%s error: %s\n  exit code %s\n  output: %s"
-					    (dwin-get self 'dotool-name)
-					    args
-					    exitcode
-					    (string-join lines)))) )
-			      (error (message "%s error: %s\n  %s"
-					      (dwin-get self 'dotool-name)
-					      args
-					      err))))))
-            ;; (cons 'dotool (lambda (&rest args)
-	    ;; 		    ;; (dwin-message 2 "%s %s" (dwin-get self 'dotool-name) args)
-            ;;                 (condition-case err
-	    ;; 			(apply #'process-lines
-	    ;; 			       (dwin-get self 'dotool-name) args)
-	    ;; 		      (error (message "%s error: args %s: %s"
-	    ;; 				      (dwin-get self 'dotool-name)
-	    ;; 				      (print1-to-string args)
-	    ;; 				      err)))))
-	    ;; public methods:
-            (cons 'switch-left  (lambda () (dwin--switch-direction self 'left)))
-            (cons 'switch-right (lambda () (dwin--switch-direction self 'right)))
-            (cons 'switch-up    (lambda () (dwin--switch-direction self 'up)))
-            (cons 'switch-down  (lambda () (dwin--switch-direction self 'down)))
-            (cons 'getactivewindow (lambda ()
-				     (let ((wins (dwin-call self
-							    'dotool
-							    "getactivewindow")))
-				       (when wins
-					 (nth 0 wins))))) ; there is always only one active window.
-            (cons 'get_desktop (lambda ()
-				 (let ((output (dwin-call self 'dotool "get_desktop")))
-				   (when output
-				     (string-to-number (nth 0 output))))))
-            (cons 'search (lambda (&rest query-args) (dwin-call-apply
-						      self 'dotool
-						      (append (list "search") query-args) )))
-            (cons 'search-class (lambda (class) (dwin-call self 'dotool
-							   "search"
-							   "--class"
-							   class)))
-            (cons 'search-pid (lambda (pid) (dwin-call self 'dotool
-						       "search" ;; "--all"
-						       "--pid"
-						       (format "%s" pid) )))
-            (cons 'getwindowname (lambda (id) (nth 0 (dwin-call
-						      self 'dotool
-						      "getwindowname" id))))
-            (cons 'getwindowgeometry (lambda (id)
-				       (let ((lines (dwin-call
-						     self 'dotool
-						     "getwindowgeometry"
-						     id)))
-					 (mapcar #'string-to-number
-						 (append
-						  (split-string
-						   (string-trim
-						    (replace-regexp-in-string
-						     "Position: " ""
-						     (nth 1 lines)))
-						   "," t)
-						  (split-string
-						   (string-trim
-						    (replace-regexp-in-string
-						     "Geometry: " ""
-						     (nth 2 lines)))
-						   "x" t))))))
-            (cons 'windowactivate (lambda (id) (dwin-call
-						self 'dotool
-						"windowactivate" id)))
-            (cons 'windowclose (lambda (id) (dwin-call self 'dotool
-						       "windowclose" id)))
-            (cons 'windowminimize (lambda (id) (dwin-call
-						self 'dotool
-						"windowminimize" id)))
-            (cons 'windowraise (lambda (id) (dwin-call
-					     self 'dotool
-					     "windowraise" id)))
-            (cons 'windowsize (lambda (id width height) (dwin-call
-							 self 'dotool
-							 "windowsize" id
-							 (format "%s" width)
-							 (format "%s" height))))
-            (cons 'windowmove (lambda (id x y &optional relative)
-				(if relative
-				    (dwin-call self 'dotool
-					       "windowmove" "--relative"
-					       id (format "%s" x)
-					       (format "%s" y))
-				  (dwin-call self 'dotool "windowmove" id
-					     (format "%s" x) (format "%s" y)))))
-            ;; (cons 'windowstate (lambda (id) (dwin-call self 'dotool
-	    ;;   "windowstate" id))) ; --add/remove/toggle <property>
-            (cons 'get_desktop_for_window (lambda (id)
-					    (string-to-number (nth 0
-					     (dwin-call self 'dotool
-							"get_desktop_for_window"
-							(format "%s" id))))))
-            (cons 'set_desktop_for_window (lambda (id number)
-					    (dwin-call self 'dotool
-						       "set_desktop_for_window"
-						       (format "%s" id)
-						       (format "%s" number))))
-            (cons 'set_desktop (lambda (number &optional relative)
-				 (if relative
-				     (dwin-call self 'dotool
-						"set_desktop" "--relative"
-						(format "%s" number))
-				   (dwin-call self 'dotool "set_desktop"
-					      (format "%s" number)))))
-	    (cons 'short-windowid (lambda (id) id)) ))
-    self))
+;;    - in dwin-kwin.el
 
 ;;_ 4.2 wm proxy for X11 generic (xdotool)
 (defun dwin-x11--window-type (window property)
@@ -528,13 +163,15 @@ TODO: break ties."
 						      "search"  ;;  "--all"
 						      "--pid"
 						      (format "%s" pid) )))
-				(dwin-collect wins pred))))
+				;; (dwin-collect wins pred))))
+				(seq-filter pred wins))))
 	  (cons 'search-class (lambda (class &optional pred)
 				"Get all X11 windows of a given CLASS that fulfill PRED."
 				(let* ((pred (or pred (dwin-get self 'search-filter)))
 				       (wins (dwin-call self 'dotool
 							"search" "--class" class)))
-				  (dwin-collect wins pred))))
+				  ;; (dwin-collect wins pred))))
+				  (seq-filter pred wins))))
 	  ;; = super.search: (as &rest args and &optional pred cannot be combined)
 	  (cons 'search-unfiltered (lambda (&rest query-args)
 			  "Get all X11 windows matching query formulated by QUERY-ARGS.
@@ -546,59 +183,11 @@ Does yield all windows without applying any filters."
 Filters using `search-filter'."
 			  (let* ((pred (dwin-get self 'search-filter))
 				 (wins (dwin-call-apply self 'search-unfiltered query-args)))
-			    (dwin-collect wins pred)))) ))))
-
+			    ;; (dwin-collect wins pred)))) ))))
+			    (seq-filter pred wins)))) ))))
 
 ;;_ 4.3 wm proxy for KDE/KWin (kdotool)
-(defun dwin--new-proxy-kwin ()
-  "Create new KWin proxy object with directional window-switching methods."
-  (let ((self (dwin--new-proxy-dotool-based)))
-    ;; further methods:
-    (dwin-extend self
-	 (list
-	  ;; overwrite attributes (new values)
-	  (cons '_class "proxy-kwin")
-	  (cons 'dotool-name "kdotool")
-	  ;; public methods:
-	  ;; "kdotool search --pid <pid>" throws odd error "Error: missing argument for option '--pid'"
-	  ;; search for two properties (--all) with an empty condition ("") for the 2nd one works.
-          (cons 'search-pid (lambda (pid) (dwin-call self 'dotool
-						     "search" "--all"
-						     "--pid" (format "%s" pid)
-						     "")))
-	  (cons 'short-windowid (lambda (id)
-				  (substring id 1 5))) ))
-    (when dwin-kwin-use-shortcuts
-      (dwin-extend self
-	 (list
-	  ;; private methods:
-	  (cons 'invoke-shortcut (lambda (name)
-				   (dwin-message 2 "invoke-shortcut: %s" name)
-				   (condition-case err
-				       (dbus-call-method
-					:session
-					"org.kde.kglobalaccel"
-					"/component/kwin"
-					"org.kde.kglobalaccel.Component"
-					"invokeShortcut"
-					name)
-				     (error (message
-					     "KWin D-Bus error: %s"
-					     err)))))
-	  ;; public methods:
-	  (cons 'switch-left  (lambda () (dwin-call self
-						    'invoke-shortcut
-						    "Switch Window Left")))
-	  (cons 'switch-right (lambda () (dwin-call self
-						    'invoke-shortcut
-						    "Switch Window Right")))
-	  (cons 'switch-up    (lambda () (dwin-call self
-						    'invoke-shortcut
-						    "Switch Window Up")))
-	  (cons 'switch-down  (lambda () (dwin-call self
-						    'invoke-shortcut
-						    "Switch Window Down"))) )))
-	 self))
+;; see dwin-kwin
 
 ;;_ 4.4 setting up the wm proxy
 (defvar dwin-proxy nil
@@ -650,7 +239,10 @@ window manager / compositor and store in `dwin-proxy'.
 Also start the Emacs server, if not running already."
   (unless (server-running-p)
     (server-start))
-  (dwin-setup-proxy))
+  (dwin-setup-proxy)
+  (unless (executable-find "_emacs-keys")
+    (dwin-install-emacs-keys-script))
+  (dwin-message 1 "dwin %s setup." (dwin-get dwin-proxy '_class)))
 
 ;;_ 4.5 convenience functions using the setup wm proxy
 (defun dwin-switch-to-desktop (number &optional relative)
@@ -658,6 +250,24 @@ Also start the Emacs server, if not running already."
 If RELATIVE is t, switch relative to the current desktop."
   (dwin-call dwin-proxy 'set_desktop number relative))
 
+(defun dwin-get-all-windows ()
+  "Get a list of all current/valid/live window ids."
+  (dwin-call dwin-proxy 'search-class ""))
+
+(defun dwin-window-alive-p (window)
+  "Return t, if WINDOW is still alive.
+A window is considered alive, if it still is reported by search."
+  (let ((windows (dwin-get-all-windows)))
+    (when (member window windows) t)))
+
+(defun dwin-filter-alive-windows (windows)
+  "Return the sublist of WINDOWS that are still alive."
+  (cl-intersection windows (dwin-get-all-windows)))
+  
+(defun dwin-sync-desktopglobal-keys ()
+  "Sync the desktopglobal keys in Emacs with the window manager."
+  (dwin-call dwin-proxy 'sync-desktopglobal-keys))
+
 
 ;;_ 5. navigation by name
 (defvar dwin-process-per-app (make-hash-table :test 'equal)
@@ -801,15 +411,27 @@ Check two sources:
 - windows created when we started CMD."
   (let* ((pids (dwin-find-pids-for-command cmd))
 	 (windows-processes (mapcan (lambda (pid) (dwin-call dwin-proxy
-							  'search-pid pid))
-				 pids))
-	 (windows-start (alist-get 'windows
-				   (gethash cmd dwin-process-per-app nil) nil))
+							     'search-pid pid))
+				    pids))
+	 (windows-start (dwin-filter-alive-windows
+			 (alist-get 'windows
+				    (gethash cmd dwin-process-per-app nil) nil)))
 	 (windows (append windows-processes windows-start)))
     (dwin-message 2 "win/cmd: '%s'\n  @proc: %s\n  @start: %s"
 		  cmd windows-processes windows-start)
     windows))
 
+(defun dwin-select-frame-set-input-focus ()
+  "Give Emacs frame focus and raise it.
+Same as `select-frame-set-input-focus', but the latter on some
+machines did not work for me."
+  (interactive)
+  (let* ((wins (dwin-call dwin-proxy 'search-pid (emacs-pid)))
+	 (window (when wins (nth 0 wins))))
+    (when window
+      (dwin-call dwin-proxy 'windowactivate window)
+      (dwin-call dwin-proxy 'windowraise window))))
+
 ;;;###autoload
 (defun dwin-switch-to-app (cmd &optional prefix)
   "Launch app CMD, if it has not been started yet.
@@ -869,7 +491,8 @@ The later function can be customized in
 `dwin-switch-to-emacs-function'."
   (interactive)
   (if (not (frame-focus-state))
-      (select-frame-set-input-focus (selected-frame))
+      ;; (select-frame-set-input-focus (selected-frame))
+      (dwin-select-frame-set-input-focus)
     (call-interactively dwin-switch-to-emacs-function)))
 
 ;;_ 6. directional navigation
